/* isdinfo.c - print ispr file information
 * Copyright (C) 2000 Tachibana Eriko
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "isd_header.h"


/* File scope variables */
static const char program[] = "isdinfo";
static const char version[] = "1.04";
static const char date[] = "2000/08/15";
static const char author[] = "Ԃq";

static char* argv0;


/* File scope function */
static void
print_kb (unsigned int n)
{
  printf ((n & 0x3ff) ? " (???)" : " (%uKB)", n >> 10);
}

/* File scope function */
static void
print_kb_ull (unsigned long long int n)
{
  printf ((n & 0x3ff) ? " (???)" : " (%LuKB)", n >> 10);
}

/* File scope function */
static void
isd_info (const char* filename, const int y_flag)
{
#define TYPE_NUM 8
  static const char* type_tbl[TYPE_NUM][2] = {
    { "3.0", "" },
    { "3.1", " (64KF k)" },
    { "3.2", " (256F k)" },
    { "3.3", " (64KF A}_[ϊ)" },
    { "3.4", " (mN 64F A}_[ϊ)" },
    { "3.5", " (YAT k)" },
    { "4.0", " (16bit PCM)" },
    { "4.5", " (16bit PCM / YAT k)" },
  };
  FILE* fp;
  IsdHeader buf;
  int filesize, frame_num;
  int type, yat;

  if (strcmp (filename, "-") == 0)
    {
      filename = "<stdin>";
      fp = stdin;
    }
  else if ((fp = fopen (filename, "rb")) == NULL)
    {
      perror (filename);
      return;
    }

  if (fread (&buf, sizeof buf, 1, fp) != 1
   || (filesize = filelength (fileno (fp))) < 0)
    {
      perror (filename);
      fclose (fp);
      return;
    }

  if (memcmp (ISPR_ID, buf.id, sizeof (buf.id)) != 0)
    {
      eprintf ("%s: ISPR f[^ł͂܂.\n", filename);
      fclose (fp);
      return;
    }

  for (type = 0; type < TYPE_NUM; ++type)
    if (memcmp (type_tbl[type][0], buf.version, sizeof (buf.version)) == 0)
      break;
  if (type >= TYPE_NUM)
    {
      eprintf ("%s: Ή̃o[Wł.\n", filename);
      fclose (fp);
      return;
    }
  /* V3.5  V4.5  YAT k */
  yat = (buf.version[2] == '5');

  printf ("t@C\t"	": %s\n", filename);
  printf ("f[^`\t"	": %s%s\n", buf.id, type_tbl[type][1]);
  printf ("𑜓x\t\t"		": %3u x %3u\n", buf.x_length, buf.y_length);
  printf ("^Ԋu\t"		": %.1ffps ` %.1ffps\n",
					60.0 / buf.interval, 60.0 / buf.interval2);

  printf ("PCM [h\t: ");
  if (buf.version[0] != '4')
    {
      static const char* freq_tbl[5] = { "3.9", "5.2", "7.8", "10.4", "15.6" };
      unsigned int freq = buf.pcm_mode >> 8;

      if ((buf.pcm_mode & 0xff) == 3 && freq <= 4)
	printf ("%skHz\n", freq_tbl[freq]);
      else
	printf ("$%08x (???)\n", buf.pcm_mode);
    }
  else
    {
      static const char* freq_tbl[8] = { "???", "16.0", "22.05", "24.0",
					 "???", "32.0", "44.1",  "48.0" };
      unsigned int freq = ((buf.pcm_mode >> 4) & 3)
			| ((buf.pcm_mode >> 5) & 4);
      const char* pan = (buf.pcm_mode & 2) ? "Stereo" : "Mono";

      printf ("%skHz (%s)\n", freq_tbl[freq], pan);
    }

  if (buf.frame_size == 0)
    buf.frame_size = buf.x_length * buf.y_length * 2;
  printf ("t[TCY\t"	": %ubyte", buf.frame_size);
  print_kb (buf.frame_size);
  printf ("\n");

  if (yat)
    {
      frame_num = buf.header_size;
      buf.header_size = 1024;		/* wb_TCY͌Œ */
    }
  else
    frame_num = (filesize - buf.header_size) / buf.frame_size;
  printf ("t[\t"	": %u\n", frame_num);

  printf ("wb_TCY\t"	": %ubyte", buf.header_size);
  print_kb (buf.header_size);
  printf ("\n");

  if (yat)
    {
      unsigned long long int s;
      unsigned int d;

      /* k̃TCY */
      s = (unsigned long long int) buf.frame_size * frame_num;
      s += buf.header_size;

      printf ("kTCY\t" ": %Lubyte", s);
      print_kb_ull (s);

      /* k */
      d = ((filesize * 1000ULL / s) + 9) / 10;
      printf (" <%2u%%>\n", d);
    }

  if (buf.cm_id[0] == 'C' && buf.cm_id[1] == 'M')
    {
      unsigned char c;
      unsigned char* p = buf.comment;
      unsigned char* end = p + sizeof buf;

      printf ("Rg\t:\n");
      c = '\n';
      while (p < end && *p)
	{
	  c = *p++;
	  if (c == '\r')
	    continue;
	  putchar (c);
	}
      if (c != '\n')
	printf ("\n");
    }

  if (yat && y_flag
   && fseek (fp, -(sizeof (unsigned long)) * frame_num, SEEK_END) == 0)
    {
      int f = buf.frame_size;
      int i;

      /* printf ("t[TCYe[u :\n"); */
      for (i = 0; i < frame_num; ++i)
	{
	  unsigned int s, d;

	  /* 1 O[hǂݍ */
	  s = fgetc (fp);
	  s = (s << 8) | fgetc (fp);
	  s = (s << 8) | fgetc (fp);
	  s = (s << 8) | fgetc (fp);

	  printf ("No.%u\t%6ubyte", i, s);
#if 0
	  print_kb (s);
#endif

	  /* k */
	  d = ((s * 1000 / f) + 9) / 10;
	  printf (" <%2u%%>\n", d);
	}
    }

  printf ("\n");

  fclose (fp);
}


/* File scope function */
static void
print_usage (void)
{
  printf ("%s version %s  %s %s.\n", program, version, date, author);
  printf ("usage: %s [option] [--] [filename] ...\n", argv0);
  printf ("option:\n"
	  "\t-y\tt[TCYe[u̓e\\n");
}

/* File scope function */
static void
print_version (void)
{
  printf ("%s version %s\n", program, version);
}


/* Function */
int
main (int argc, char* argv[])
{
  int i;
  int count;
  int no_opt, y_flag;

  argv0 = argv[0];

  no_opt = 0;
  y_flag = 0;
  count = 0;

  for (i = 1; i < argc; ++i)
    {
      char* p = argv[i];

      if (*p == '\0')
	continue;

      if (*p++ != '-' || *p == '\0' || no_opt)
	{
	  isd_info (--p, y_flag);
	  ++count;
	}

      else if (p[0] == '-' && p[1] == '\0')
	no_opt = 1;			/* -- */

      else if (p[0] == 'y' && p[1] == '\0')
	y_flag = 1;			/* -y */

      else if (*p == '?' || *p == 'h' || *p == 'H' || strcmp (p, "-help") == 0)
	{
	  print_usage ();		/* gp@\ */
	  return EXIT_SUCCESS;
	}

      else if (*p == 'v' || *p == 'V' || strcmp (p, "-version") == 0)
	{
	  print_version ();		/* o[W\ */
	  return EXIT_SUCCESS;
	}

      else
	{
	  print_usage ();		/* sȈ */
	  return EXIT_FAILURE;
	}
    }

  if (count == 0)
    {
      if (isatty (STDIN_FILENO))
	{
	  print_usage ();		/* ȗ */
	  return EXIT_FAILURE;
	}
      else
	isd_info ("-", y_flag);
    }

  return EXIT_SUCCESS;
}

/* EOF */
